//---------------------------------------------------------------------------
#include <vcl.h>
#include <FileCtrl.hpp>
#include <Math.hpp>
#pragma hdrstop

#include "Unit1.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TForm1 *Form1;
//---------------------------------------------------------------------------
__fastcall TForm1::TForm1(TComponent* Owner)
    : TForm(Owner)
{
}
//---------------------------------------------------------------------------
int TForm1::DeleteFile(String fileName, bool allowUndo)
{
    SHFILEOPSTRUCT shop;
    ZeroMemory(&shop, sizeof(shop));
    shop.hwnd = Form1->Handle;
    shop.wFunc = FO_DELETE;             // Wanna delete file
    shop.pFrom = fileName.c_str();      // fileName that wanna be deleted, double Null terminated.
    shop.pTo = NULL;
    shop.fFlags = FOF_FILESONLY | FOF_NOCONFIRMATION |  FOF_NO_CONNECTED_ELEMENTS;
    if(allowUndo)
        shop.fFlags |= FOF_ALLOWUNDO;   // Delete to Recycle Bin
    return(SHFileOperation(&shop));
}
//---------------------------------------------------------------------------
void TForm1::FindFilesInDirectory(String directory, String extension, bool searchSubdir)
{
    WIN32_FIND_DATA filedata ;  // Structure for file data
    HANDLE filehandle ;         // Handle for searching

    if( extension == "" )
        extension = "*.*";
    // Pass 1 - Search for the files within the directory.
    filehandle = FindFirstFile((directory + extension).c_str(), &filedata) ;
    if( filehandle != INVALID_HANDLE_VALUE )
    {
        do
        {
            if( (filedata.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) == 0 )
            {
                // Display the file name.
                TListItem *li = ListView1->Items->Add();
                li->Caption = directory + filedata.cFileName ;

                // Display the file size.
                if( filedata.nFileSizeHigh == 0 )
                {
                    int filesize = filedata.nFileSizeLow + 1023;
                    if( filesize >= 1073741824 )      // 1024^3
                        li->SubItems->Add(String(filesize / 1073741824) + " GB") ;
                    else if( filesize >= 1048576 )    // 1024^2
                        li->SubItems->Add(String(filesize / 1048576) + " MB") ;
                    else
                        li->SubItems->Add(String(filesize / 1024) + " KB") ;
                    // Add filesize to SubItems, for sorting file size purpose.
                    li->SubItems->Add(String(filesize));
                }
                else
                {
                    // If nFileSizeHigh is non-zero then this is a **HUGE** file.
                    double filesize = (MAXDWORD * filedata.nFileSizeHigh) + filedata.nFileSizeLow;
                    if( filesize >= 1073741824.0 )      // 1024^3
                        li->SubItems->Add(String(filesize / 1073741824.0) + " GB") ;
                    else if( filesize >= 1048576.0 )    // 1024^2
                        li->SubItems->Add(String(filesize / 1048576.0) + " MB") ;
                    else
                        li->SubItems->Add(String(filesize / 1024.0) + " KB") ;
                    // Add filesize to SubItems, for sorting file size purpose.
                    li->SubItems->Add(String(filesize));
                }
                Application->ProcessMessages() ;
            }
        } while(FindNextFile(filehandle, &filedata)) ;
        FindClose(filehandle) ;
    }

    if(searchSubdir)    // If user wanna search subdirectories..
    {
        // Pass 2 - Search for all the subdirectories within this directory
        filehandle = FindFirstFile ((directory + "*.*").c_str (), &filedata) ;
        if( filehandle != INVALID_HANDLE_VALUE)
        {
            do
            {
                if( (filedata.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) != 0 &&
                    String (filedata.cFileName) != "." && String (filedata.cFileName) != "..")
                {
                    // Recursive call here
                    FindFilesInDirectory(directory + filedata.cFileName + "\\", extension, true) ;
                }
                Application->ProcessMessages() ;
            } while(FindNextFile(filehandle, &filedata)) ;
            FindClose(filehandle) ;
        }
        Application->ProcessMessages() ;
    }
}
//---------------------------------------------------------------------------
void __fastcall TForm1::FormCreate(TObject *Sender)
{
    edtDir->Text = ExtractFilePath(Application->ExeName);
    iniSetting = new TIniFile(ExtractFilePath(Application->ExeName) + "Setting.ini");
    cbx1->Checked = iniSetting->ReadBool("SearchType", "*.*~", true);
    cbx2->Checked = iniSetting->ReadBool("SearchType", "*.il?", true);
    cbx3->Checked = iniSetting->ReadBool("SearchType", "*.tds", true);
    cbx4->Checked = iniSetting->ReadBool("SearchType", "*.obj", true);
    cbx5->Checked = iniSetting->ReadBool("SearchType", "*.bak", true);
    cbx6->Checked = iniSetting->ReadBool("SearchType", "*.dsk", false);
    cbx7->Checked = iniSetting->ReadBool("SearchType", "*.~*", true);
    cbxSubdir->Checked = iniSetting->ReadBool("SearchType", "SearchSubdir", true);
    cbxCustom->Checked = iniSetting->ReadBool("SearchType", "Custom", false);
    edtType->Text = iniSetting->ReadString("SearchType", "CustomType", "*.*");
    cbxRecycle->Checked = iniSetting->ReadBool("Recycle", "allowUndo", true);
}
//---------------------------------------------------------------------------
void __fastcall TForm1::FormDestroy(TObject *Sender)
{
    iniSetting->WriteBool("SearchType", "*.*~", cbx1->Checked);
    iniSetting->WriteBool("SearchType", "*.il?", cbx2->Checked);
    iniSetting->WriteBool("SearchType", "*.tds", cbx3->Checked);
    iniSetting->WriteBool("SearchType", "*.obj", cbx4->Checked);
    iniSetting->WriteBool("SearchType", "*.bak", cbx5->Checked);
    iniSetting->WriteBool("SearchType", "*.dsk", cbx6->Checked);
    iniSetting->WriteBool("SearchType", "*.~*", cbx7->Checked);
    iniSetting->WriteBool("SearchType", "SearchSubdir", cbxSubdir->Checked);
    iniSetting->WriteBool("SearchType", "Custom", cbxCustom->Checked);
    iniSetting->WriteString("SearchType", "CustomType", edtType->Text);
    iniSetting->WriteBool("Recycle", "allowUndo", cbxRecycle->Checked);
    delete iniSetting;
}
//---------------------------------------------------------------------------
void __fastcall TForm1::FormKeyPress(TObject *Sender, char &Key)
{
    if( Key == 27 )
        Close();
}
//---------------------------------------------------------------------------
void __fastcall TForm1::btnSelectDirClick(TObject *Sender)
{
    AnsiString dir;
    if(SelectDirectory("пܤ@ӭnjMƧ.", "", dir))
        edtDir->Text = dir;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::btnSearchClick(TObject *Sender)
{
    if( edtDir->Text.Trim() == "" )
    {
        ShowMessage("ХܭnjMƧ !");
        return;
    }
    edtDir->Text = edtDir->Text.Trim();
    if( !(cbxCustom->Checked | cbx1->Checked | cbx2->Checked | cbx3->Checked |
          cbx4->Checked | cbx5->Checked | cbx6->Checked | cbx7->Checked))
    {
        ShowMessage("Цܤֿ@حnjMɮ !");
        return;
    }

    ListView1->Clear();
    btnSelectDir->Enabled = false;
    btnSearch->Enabled = false;
    btnSelectAll->Enabled = false;
    btnSelectNone->Enabled = false;
    btnDelete->Enabled = false;
    isDesc = 1;

    ListView1->Items->BeginUpdate();
    // Check if file path is end with a backslash
    if( edtDir->Text.LastDelimiter("\\") != edtDir->Text.Length() )
        edtDir->Text = edtDir->Text + "\\";
    if( cbxCustom->Checked )
    {
        // Check if multiple searching file types
        if( edtType->Text.AnsiPos(";") == 0 )
            FindFilesInDirectory(edtDir->Text, edtType->Text.Trim(), cbxSubdir->Checked);
        else
        {
            TStringList *sl = new TStringList;
            ExtractStrings(TSysCharSet() << ';', TSysCharSet() << ' ', edtType->Text.c_str(), sl);
            for( int i = 0; i < sl->Count; i++ )
                FindFilesInDirectory(edtDir->Text, sl->Strings[i].Trim(), cbxSubdir->Checked);
            delete sl;
        }
    }
    if( cbx1->Checked )
        FindFilesInDirectory(edtDir->Text, cbx1->Caption, cbxSubdir->Checked);
    if( cbx2->Checked )
        FindFilesInDirectory(edtDir->Text, cbx2->Caption, cbxSubdir->Checked);
    if( cbx3->Checked )
        FindFilesInDirectory(edtDir->Text, cbx3->Caption, cbxSubdir->Checked);
    if( cbx4->Checked )
        FindFilesInDirectory(edtDir->Text, cbx4->Caption, cbxSubdir->Checked);
    if( cbx5->Checked )
        FindFilesInDirectory(edtDir->Text, cbx5->Caption, cbxSubdir->Checked);
    if( cbx6->Checked )
        FindFilesInDirectory(edtDir->Text, cbx6->Caption, cbxSubdir->Checked);
    if( cbx7->Checked )
        FindFilesInDirectory(edtDir->Text, cbx7->Caption, cbxSubdir->Checked);
    ListView1->Items->EndUpdate();

    btnSearch->Enabled = true;
    btnSelectDir->Enabled = true;

    if( ListView1->Items->Count > 0 )
    {
        btnSelectAll->Enabled = true;
        btnSelectNone->Enabled = true;
        btnDelete->Enabled = true;
    }
}
//---------------------------------------------------------------------------
void __fastcall TForm1::ListView1ColumnClick(TObject *Sender,
      TListColumn *Column)
{
    isDesc = -isDesc;
    ListView1->Tag = Column->Index ;
    ListView1->CustomSort(NULL, isDesc);
}
//---------------------------------------------------------------------------
void __fastcall TForm1::ListView1Compare(TObject *Sender, TListItem *Item1,
      TListItem *Item2, int Data, int &Compare)
{
    if( ListView1->Tag == 0 )   // first column N caption
        Compare = CompareText(Item1->Caption, Item2->Caption);
    else
        Compare = CompareValue(StrToFloat(Item1->SubItems->Strings[ListView1->Tag]),
                               StrToFloat(Item2->SubItems->Strings[ListView1->Tag]));
    if( Data == 1 ) // descending
        Compare = -Compare;
}
//---------------------------------------------------------------------------
void __fastcall TForm1::btnSelectAllClick(TObject *Sender)
{
    ListView1->SetFocus();
    ListView1->SelectAll();
}
//---------------------------------------------------------------------------
void __fastcall TForm1::btnSelectNoneClick(TObject *Sender)
{
    ListView1->SetFocus();
    ListView1->ClearSelection();
}
//---------------------------------------------------------------------------
void __fastcall TForm1::btnDeleteClick(TObject *Sender)
{
    btnSelectDir->Enabled = false;
    btnSearch->Enabled = false;
    btnSelectAll->Enabled = false;
    btnSelectNone->Enabled = false;
    btnDelete->Enabled = false;

    ListView1->Items->BeginUpdate();
    while(ListView1->Selected)
    {
        // File path must be "double Null terminated", so we add '\0' at the end.
        if(DeleteFile(ListView1->Selected->Caption+'\0', cbxRecycle->Checked) == 0)
            ListView1->Selected->Delete();
        else
            break;
    }
    ListView1->Items->EndUpdate();

    btnSelectDir->Enabled = true;
    btnSearch->Enabled = true;
    if( ListView1->Items->Count == 0 )
    {
        btnSelectAll->Enabled = false;
        btnSelectNone->Enabled = false;
    }
    else
        btnDelete->Enabled = true;
}
//---------------------------------------------------------------------------
